! *****************************************************************************
! *   propergave wave by time integration                                     *
! *****************************************************************************

   subroutine propag( f, vel, d, u, c, fnew, fstr, g, gnew, gstr )
   
      use m_param
      
      implicit none
      real(8):: f(-1:mx1)
      real(8):: vel(-1:mx1)
      real(8):: d(-1:mx1)
      real(8):: u(-1:mx1)
      real(8):: c(-1:mx1)
      real(8):: fnew(-1:mx1), fstr(-1:mx1)
      real(8):: g(-1:mx1)
      real(8):: gnew(-1:mx1), gstr(-1:mx1)
      integer:: i, iB      

!  ****  non-advection term (bathymetric effect)  ****

      do i = 0,mx
        fstr(i) = f(i) - dt * 0.5d0 * ga / c(i) *  0.5d0 / dx *            &
                                     ( u(i+1) * d(i+1) - u(i-1) * d(i-1) )
      end do
      
!  **** boundary condition  ****

      iB = -1
      if ( vel(iB) < 0.0d0 ) then
        fstr(iB) = f(iB) - dt * 0.5d0 * ga / c(iB) / dx *                  &
                                ( u(iB+1) * d(iB+1) - u(iB) * d(iB) ) 
       else
        fstr(iB) = f(iB)
      end if 

      iB = mx1
      if ( vel(iB) > 0.0d0 ) then
        fstr(iB) = f(iB) - dt * 0.5d0 * ga / c(iB) / dx *                  &
                                ( u(iB) * d(iB) - u(iB-1) * d(iB-1) ) 
       else
        fstr(iB) = f(iB)
      end if 

!  ****  non-advection term for gradient equation  ****

      do i = 1,mx
        gstr(i) = g(i) + 0.5d0 / dx *                                      &
                         ( fstr(i+1) - f(i+1) - (fstr(i-1) - f(i-1) ) ) 
      end do

!  ****  Boundary condition  ****

      iB = -1
      if ( vel(iB) < 0.0d0 ) then
        gstr(iB) = g(iB) + ( fstr(iB+1) - f(iB+1) - (fstr(iB) - f(iB) ) ) / dx
       else
        gstr(iB) = g(iB)
      end if
      iB = mx1
      if ( vel(iB) < 0.0d0 ) then
        gstr(iB) = g(iB) + ( fstr(iB) - f(iB) - (fstr(iB-1) - f(iB-1) ) ) / dx
       else
        gstr(iB) = g(iB)
      end if

!  ****  advection term  ****

      call advec_1D ( fstr, gstr, vel, fnew, gnew )
      call set_bound( fstr, gstr, vel, fnew, gnew )

!  ****  change old and new time-step values  ****

      f(:) = fnew(:)
      g(:) = gnew(:)    

      return

   end subroutine propag
