! *****************************************************************************
! **  1-D wave propergation in the shallow water                             **
! **                                                                         **
! *****************************************************************************

   program wave_main
   
      use m_param
      
      implicit none
      real(8), allocatable:: x(:),     d(:)
      real(8), allocatable:: u(:),     h(:)
      real(8), allocatable:: c(:),     v(:),     w(:)
      real(8), allocatable:: p(:),     q(:)
      real(8), allocatable:: pnew(:),  qnew(:)
      real(8), allocatable:: pstr(:),  qstr(:)
      real(8), allocatable:: px(:),    qx(:)
      real(8), allocatable:: pxnew(:), qxnew(:)
      real(8), allocatable:: pxstr(:), qxstr(:)
      real(8):: t
      integer:: itr, nf

!  ****  set model parameters  ****

      call set_param

!  ****  set grid  ****

      dx  = xL / dble( mx )
      mx1 = mx+1
!      write ( 6,* ) mx,mx1,dx
      
!  ****  allocate dimension variables  ****

      allocate ( x(-1:mx1), d(-1:mx1) )
      allocate ( u(-1:mx1), h(-1:mx1) )
      allocate ( c(-1:mx1), v(-1:mx1), w(-1:mx1) )
      allocate ( p(-1:mx1), q(-1:mx1) )
      allocate ( pnew(-1:mx1), qnew(-1:mx1) )
      allocate ( pstr(-1:mx1), qstr(-1:mx1) )
      allocate ( px(-1:mx1), qx(-1:mx1) )
      allocate ( pxnew(-1:mx1), qxnew(-1:mx1) )
      allocate ( pxstr(-1:mx1), qxstr(-1:mx1) )

!  ****  set bathymetry of ocean bottom  ****

      call set_bathy( d )

!  ****  set initial conditions  ****

      itr = 0
      nf  = 0
      t = 0.0d0

      call set_init( x, u, h )

!  ****  convert variables  *****

      call uh_to_cvwpq( u, h, c, v, w, p, q )

!  ****  write inital conditions to file  ****

      call uh_to_file( x, u, h, nf )
!      call pq_to_file( x, p, q, nf )

!  ****  Check Courant number  ****

      call chk_cour( v, w )

      open ( 18,file='tic.dat' )
      write( 18,* ) itr,t,cvw
      write( 6,* )  'main',itr,t,cvw
      
!  ****  set inital gradient values for CIP method  ****

      call set_igrad( p, px )
      call set_igrad( q, qx )  

!   ========================  time integration loop  =========================================

      do itr = 1,ntmax

        t = t + dt
!        write ( 6,* ) 'Before progress of p'
        call propag( p, v, d, u, c, pnew, pstr, px, pxnew, pxstr )
!        write ( 6,* ) 'Before progress of q'
        call propag( q, w, d, u, c, qnew, qstr, qx, qxnew, qxstr )
        
        call pq_to_cuhvw( p, q, c, u, h, v, w )

!  ****  write inital conditions to file  ****

        if ( mod( itr, ifskp )==0 ) then
          nf = nf + 1
          call uh_to_file( x, u, h, nf  )
!          call pq_to_file( x, p, q, nf )
        end if

        call chk_cour( v, w )
        write( 18,* ) itr,t,cvw
        write( 6,* )  'main',itr,t,cvw
        
      end do

!   ========================  loop end  ======================================================

      close ( 10 ) 
      close ( 18 ) 
      
   end program wave_main
